@@ -1,8 +1,9 @@
 use crate::{money::Money, CheckoutError};
 use async_trait::async_trait;
 use serde::{Deserialize, Serialize};
+use schemars::JsonSchema;
 
-#[derive(Clone, Serialize, Deserialize)]
+#[derive(Clone, Serialize, Deserialize, JsonSchema)]
 pub struct Item {
     pub sku: String,
     pub title: String,
