@@ -5,10 +5,11 @@ use crate::money::{Currency, Money};
 use address::Address;
 use async_trait::async_trait;
 pub use chrono::{DateTime, Utc};
-use inventory::StockIssue;
+use inventory::{ReserveResult, StockIssue};
 use invoice::{Invoice, InvoiceCalculator};
 use item::Item;
 use price::PriceCalculator;
+use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 use shipping::{FulfillmentType, FulfillmentTypeSelection, ShippingCalculator, ShippingQuote};
 use std::matches;
@@ -24,7 +25,7 @@ pub mod price;
 pub mod server;
 pub mod shipping;
 
-#[derive(Serialize, Deserialize)]
+#[derive(Serialize, Deserialize, JsonSchema)]
 pub enum State {
     Shopping,
     ItemsConfirmed(DateTime<Utc>),
@@ -32,13 +33,14 @@ pub enum State {
     Abandoned,
 }
 
-#[derive(Serialize, Deserialize)]
+#[derive(Serialize, Deserialize, JsonSchema)]
 pub struct Checkout {
     pub id: Uuid,
     pub state: State,
     pub currency: Currency,
     pub promo_codes: Vec<String>,
     pub items: Vec<Item>,
+    pub stock_issues: Vec<StockIssue>,
     pub fulfillment_type: Option<FulfillmentType>,
     pub shipping_address: Option<Address>,
     pub shipping_quotes: Vec<ShippingQuote>,
@@ -65,37 +67,6 @@ impl Checkout {
         }
     }
 
-    async fn enter_items_confirmed_state<C: CheckoutContext + Send>(
-        &mut self,
-        ctx: &mut C,
-    ) -> Result<(), CheckoutError> {
-        match self.state {
-            State::ItemsConfirmed(_) => Ok(()),
-            State::Shopping => {
-                self.fulfillment_type = None;
-                if let Some(address) = &self.shipping_address {
-                    self.shipping_quotes = ctx
-                        .get_shipping_quotes(
-                            &self.currency,
-                            &self.promo_codes,
-                            &self.items,
-                            address,
-                        )
-                        .await?;
-                }
-
-                ctx.update_item_prices(&self.currency, &self.promo_codes, &mut self.items)
-                    .await?;
-                self.update_invoice(ctx).await?;
-
-                Ok(self.state = State::ItemsConfirmed(ctx.reserve_items(&self.items).await?))
-            }
-            _ => Err(new_invalid_state_error(
-                "must be in the shopping or items_confirmed state",
-            )),
-        }
-    }
-
     async fn update_invoice<C: CheckoutContext + Send>(
         &mut self,
         ctx: &mut C,
@@ -113,6 +84,7 @@ impl Checkout {
             currency,
             promo_codes: vec![],
             items: vec![],
+            stock_issues: vec![],
             fulfillment_type: None,
             shipping_address: None,
             shipping_quotes: vec![],
@@ -191,9 +163,41 @@ impl Checkout {
         &mut self,
         ctx: &mut C,
     ) -> Result<(), CheckoutError> {
-        self.enter_items_confirmed_state(ctx).await?;
+        match self.state {
+            State::ItemsConfirmed(_) => Ok(()),
+            State::Shopping => {
+                match ctx.reserve_items(&self.items).await? {
+                    ReserveResult::Success(deadline) => {
+                        self.fulfillment_type = None;
+                        if let Some(address) = &self.shipping_address {
+                            self.shipping_quotes = ctx
+                                .get_shipping_quotes(
+                                    &self.currency,
+                                    &self.promo_codes,
+                                    &self.items,
+                                    address,
+                                )
+                                .await?;
+                        }
 
-        ctx.on_confirm_items(self).await
+                        ctx.update_item_prices(&self.currency, &self.promo_codes, &mut self.items)
+                            .await?;
+                        self.update_invoice(ctx).await?;
+                        self.state = State::ItemsConfirmed(deadline);
+                        self.stock_issues = vec![];
+                        ctx.on_confirm_items(self).await?;
+                    }
+                    ReserveResult::StockIssue(stock_issues) => {
+                        self.stock_issues = stock_issues;
+                    }
+                }
+
+                Ok(())
+            }
+            _ => Err(new_invalid_state_error(
+                "must be in the shopping or items_confirmed state",
+            )),
+        }
     }
 
     pub async fn update_shipping_address<C: CheckoutContext + Send>(
@@ -260,6 +264,18 @@ impl Checkout {
 pub trait CheckoutContext:
     InvoiceCalculator + ItemTitleResolver + InventoryController + ShippingCalculator + PriceCalculator
 {
+    async fn start_transaction(&mut self) -> Result<(), CheckoutError> {
+        Ok(())
+    }
+
+    async fn commit_transaction(&mut self) -> Result<(), CheckoutError> {
+        Ok(())
+    }
+
+    async fn abort_transaction(&mut self) -> Result<(), CheckoutError> {
+        Ok(())
+    }
+
     async fn on_create(&mut self, _co: &Checkout) -> Result<(), CheckoutError> {
         Ok(())
     }
@@ -295,11 +311,15 @@ pub trait CheckoutContext:
     }
 }
 
+#[async_trait]
+pub trait CheckoutContextProvider {
+    async fn new_context<T: CheckoutContext>(&self) -> Result<T, CheckoutError>;
+}
+
 #[derive(Debug, Serialize, Deserialize)]
 pub enum CheckoutError {
     InvalidState(Error),
     BadRequest(Error),
-    StockIssue(Vec<StockIssue>),
 }
 
 fn new_invalid_state_error(msg: &str) -> CheckoutError {
